; send cmd to lcd
; B = cmd
lcd_send_cmd:
	call i2c_start
	mvi a,lcd_slave
	call i2c_write
	call get_ack
	mov a,b
	rlc
	rlc
	rlc
	rlc
	ani 0f0h
	mov c,a	; cmd_l
	mov a,b
	ani 0f0h
	mov d,a	; cmd_u
	ori 0ch
	call i2c_write
	call get_ack
	mov a,d
	ori 08h
	call i2c_write
	call get_ack
	mov a,c
	ori 0ch
	call i2c_write
	call get_ack
	mov a,c
	ori 08h
	call i2c_write
	call get_ack
	call i2c_stop
	ret

; send data to lcd
; B = data
lcd_send_data:
	call i2c_start
	mvi a,lcd_slave
	call i2c_write
	call get_ack
	mov a,b
	rlc
	rlc
	rlc
	rlc
	ani 0f0h
	mov c,a	; cmd_l
	mov a,b
	ani 0f0h
	mov d,a	; cmd_u
	ori 0dh
	call i2c_write
	call get_ack
	mov a,d
	ori 09h
	call i2c_write
	call get_ack
	mov a,c
	ori 0dh
	call i2c_write
	call get_ack
	mov a,c
	ori 09h
	call i2c_write
	call get_ack
	call i2c_stop
	ret

lcd_init:
	mvi b,02h		; return home
	call lcd_send_cmd
	mvi b,028h		; 4-bit mode
	call lcd_send_cmd
	mvi b,0ch		; Display On, Cursor off
	call lcd_send_cmd
	mvi b,06h		; Increment cursor (shift cursor to right)
	call lcd_send_cmd
	mvi b,01h		; Clear Display
	call lcd_send_cmd
	ret

get_ack:	; Get ACK from i2c slave
    	mvi A,PIO_C_IN			; SDA - in, SCL - out 
	out PIO_M		; SDA goes HI as its set to input,
	call sclset			; raised CLK, now expect "low" on SDA as the sign on ACK	
	in PIO_M	 	; here read SDA and look for "LOW" = ACK, "HI" - NOACK or Timeout`
	call sclclr
	ret
	; ToDo - implement the ACK timeout, right now we blindly assume the ACK came in.

send_ack: 
	mvi a,PIO_C_OUT	; SDA + SCL output
	out PIO_M	;
	call sclset		; Clock SCL
	call sclclr
	ret

send_noack:		; Send NAK (no ACK) to i2c bus (master keeps SDA HI on the 9th bit of data)
	mvi a,PIO_C_OUT     		
	out PIO_M		
	call sdaset			; 	
	call sclset			; Clock SCL 
	call sclclr
	ret
	


getbyte:	; Read 8 bits from i2c bus
        push b
		mvi A,PIO_C_IN       		; SDA - in, SCL - out
		out PIO_M   		;
		mvi b,8
gb1:    call    sclset          ; Clock UP
		in PIO_C			; SDA (RX data bit) is in A.0
		rrc					; move RX data bit to CY
		ral                    	; Shift CY into C
		mov c,a
        call    sclclr          ; Clock DOWN
	dcr b
        jnz    gb1
        mov a,c             		; Return RX Byte in A
	pop b
        ret


i2c_write: 	; Send byte from A to i2C bus

        push    b
        mov      c,a             ;Shift register

	mvi a,PIO_C_OUT	; SDA + SCL output
	out PIO_M	;

        mvi      b,8
pbks1:  mov a,c
	rlc                    ;B[7] => CY
	mov c,a
        call    sdaput          ; & so to SDA
        call    sclclk          ;Clock it
	dcr b
        jnz    pbks1
        call    sdaset          ;Leave SDA high, for ACK
        pop     b
        ret


sclclk:         ;	"Clock" the SCL line Hi -> Lo
	call    sclset
	call    sclclr
	ret

sdaput:        ; CY state copied to SDA line, without changing SCL state
        jnc	sdz
	mvi a,(SDA_Bit << 1) | 1	; set bit
	out PIO_M
        ret
sdz:
	mvi a,(SDA_Bit << 1) & 0FEh	; clear bit
	out PIO_M
        ret

i2c_stop:           ; i2c STOP sequence, SDA goes HI while SCL is HI
        push    psw
	mvi a,PIO_C_OUT
	out PIO_M		
        call    sdaclr
        call    sclset
        call    sdaset
        pop     psw
        ret

i2c_start:          ; i2c START sequence, SDA goes LO while SCL is HI
	mvi a,PIO_C_OUT
	out PIO_M		
	call	sdaset
	call    sclset
	call    sdaclr
	call    sclclr
	call    sdaset
	ret

sclset: ; SCL HI without changing SDA     	
	mvi a,(SCL_Bit << 1) | 1	; set bit
	out PIO_M

        ;in      PIO_C
	;ori 	1 << SCL_bit
        ;out     PIO_C
        ret

sclclr:  ; SCL LO without changing SDA       	
	mvi a,(SCL_Bit << 1) & 0FEh	; clear bit
	out PIO_M

        ;in      PIO_C
	;ani 	~(1 << SCL_bit) ;0EFh
        ;out     PIO_C
        ret

sdaset:	; SDA HI without changing SCL
	mvi a,(SDA_Bit << 1) | 1	; set bit
	out PIO_M

        ;in      PIO_C
	;ori	1 << SDA_bit
        ;out     PIO_C
        ret

sdaclr: ; SDA LO without changing SCL   	
	mvi a,(SDA_Bit << 1) & 0FEh	; clear bit
	out PIO_M

        ;in      PIO_C
	;ani 	~(1 << SDA_bit) ;0FEh
        ;out     PIO_C
        ret
